package vue;

// Java emulation core implementation
class JavaVUE extends VUE {

    // Instance fields
    byte[] rom;  // Cartridge ROM
    byte[] sram; // Cartridge SRAM
    byte[] wram; // System WRAM

    // Components
    Bus bus; // Memory bus



    ///////////////////////////////////////////////////////////////////////////
    //                               Constants                               //
    ///////////////////////////////////////////////////////////////////////////

    // Memory access type sizes
    static final int[] TYPE_SIZES = { 1, 1, 2, 2, 4 };



    ///////////////////////////////////////////////////////////////////////////
    //                             Constructors                              //
    ///////////////////////////////////////////////////////////////////////////

    // Default constructor
    JavaVUE() {
        bus  = new Bus(this);
        wram = new byte[0x10000];
    }



    ///////////////////////////////////////////////////////////////////////////
    //                            Public Methods                             //
    ///////////////////////////////////////////////////////////////////////////

    // Release any used resources
    public void dispose() { }; // No action needed

    // Retrieve a copy of the ROM data
    public byte[] getROM() {
        var ret = new byte[rom.length];
        System.arraycopy(rom, 0, ret, 0, ret.length);
        return ret;
    }

    // Determine whether the context is native-backed
    public boolean isNative() {
        return false;
    }

    // Read bytes from the CPU bus
    public boolean read(int address, byte[] dest, int offset, int length) {
        int count = 0; // Bytes to read in one iteration

        // Error checking
        if (
            dest   == null ||
            offset <  0    ||
            length <  0    ||
            offset + length > dest.length
        ) return false;

        // Perform the operation
        for (; length > 0; address += count, length -= count, offset += count){

            // Determine the maximum number of bytes to process at once
            count = Math.min(length, 0x01000000 - (address & 0x00FFFFFF));

            // Process by component
            switch (address >> 24 & 7) {
                case 1: // VSU
                    // Will support debug reads
                    bus.readBytes(null, dest, address, 0     , count);
                    break;
                case 3: // Fallthrough, unmapped
                case 4: // Cartridge expansion (not supported)
                    bus.readBytes(null, dest, address, 0     , count);
                    break;
                case 5: /* System WRAM */
                    bus.readBytes(wram, dest, address, offset, count);
                    break;
                case 6: /* Cartridge RAM */
                    bus.readBytes(sram, dest, address, offset, count);
                    break;
                case 7: /* Cartridge ROM */
                    bus.readBytes(rom , dest, address, offset, count);
                    break;
            }

        };
        return true;
    }

    // Provide new ROM data
    public boolean setROM(byte[] data, int offset, int length) {

        // Error checking
        if (
            data == null  ||
            offset < 0    ||
            length < 1024 || length > 0x01000000 ||
            (length & length - 1) != 0           ||
            offset + length > data.length
        ) return false;

        // Accept the new ROM data
        rom = new byte[length];
        System.arraycopy(data, offset, rom, 0, length);
        return true;
    }

    // Write bytes to the CPU bus
    public boolean write(int address, byte[] src, int offset, int length) {
        int count = 0; // Bytes to process in one iteration

        // Error checking
        if (
            src    == null ||
            offset <  0    ||
            length <  0    ||
            offset + length > src.length
        ) return false;

        // Perform the operation
        for (; length > 0; address += count, length -= count, offset += count){

            // Determine the maximum number of bytes to process at once
            count = Math.min(length, 0x01000000 - (address & 0x00FFFFFF));

            // Process by component
            switch (address >> 24 & 7) {
                case 5: /* System WRAM */
                    bus.writeBytes(wram, src, address, offset, count);
                    break;
                case 6: /* Cartridge RAM */
                    bus.writeBytes(sram, src, address, offset, count);
                    break;
                case 7: /* Cartridge ROM */
                    bus.writeBytes(rom , src, address, offset, count);
                    break;
            }

        };
        return true;
    }

}
